package com.yonge.cooleshow.biz.dal.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.dao.StudentDao;
import com.yonge.cooleshow.biz.dal.dao.SubjectDao;
import com.yonge.cooleshow.biz.dal.dao.TeacherDao;
import com.yonge.cooleshow.biz.dal.dao.UserBindingTeacherDao;
import com.yonge.cooleshow.biz.dal.dto.search.QueryMyFollowSearch;
import com.yonge.cooleshow.biz.dal.dto.search.StudentSearch;
import com.yonge.cooleshow.biz.dal.entity.Student;
import com.yonge.cooleshow.biz.dal.entity.StudentTotal;
import com.yonge.cooleshow.biz.dal.entity.Subject;
import com.yonge.cooleshow.biz.dal.entity.UserBindingTeacher;
import com.yonge.cooleshow.biz.dal.enums.ClientEnum;
import com.yonge.cooleshow.biz.dal.mapper.SysUserMapper;
import com.yonge.cooleshow.biz.dal.service.ImUserFriendService;
import com.yonge.cooleshow.biz.dal.service.StudentService;
import com.yonge.cooleshow.biz.dal.service.StudentTotalService;
import com.yonge.cooleshow.biz.dal.vo.MyFollow;
import com.yonge.cooleshow.biz.dal.vo.StudentHomeVo;
import com.yonge.cooleshow.biz.dal.vo.StudentVo;
import com.yonge.cooleshow.biz.dal.vo.TeacherVo;
import com.yonge.cooleshow.biz.dal.wrapper.StudentWrapper;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.enums.CacheNameEnum;
import com.yonge.cooleshow.common.enums.UserLockFlag;
import com.yonge.toolset.base.util.StringUtil;
import com.yonge.toolset.utils.date.DateUtil;
import com.yonge.toolset.utils.string.ValueUtil;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RMap;
import org.redisson.api.RedissonClient;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.MessageFormat;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.TimeUnit;


@Service
public class StudentServiceImpl extends ServiceImpl<StudentDao, Student> implements StudentService {
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private StudentTotalService totalService;
    @Autowired
    private ImUserFriendService imUserFriendService;
    @Resource
    private UserBindingTeacherDao userBindingTeacherDao;
    @Resource
    private TeacherDao teacherDao;
    @Autowired
    private SysUserMapper sysUserMapper;
    @Autowired
    private SubjectDao subjectDao;

    @Override
    public StudentVo detail(Long userId) {
        return baseMapper.detail(userId);
    }

    @Override
    public StudentVo detailByPhone(String phone) {
        return baseMapper.detailByPhone(phone);
    }

    @Override
    public IPage<StudentVo> selectPage(IPage<StudentVo> page, StudentSearch studentSearch) {
        return page.setRecords(baseMapper.selectPage(page, studentSearch));
    }

    @Override
    public StudentHomeVo queryUserInfo(SysUser user) {
        StudentVo detail = detail(user.getId());
        StudentHomeVo studentHomeVo = new StudentHomeVo();
        BeanUtils.copyProperties(detail, studentHomeVo);

        //身份证号、手机号脱敏
        studentHomeVo.setIdCardNo(ValueUtil.fuzzyIdCard(studentHomeVo.getIdCardNo()));
        studentHomeVo.setPhone(ValueUtil.fuzzyMobile(studentHomeVo.getPhone()));
        studentHomeVo.setHeardUrl(user.getAvatar());

        int num = DateUtil.daysBetween(new Date(), detail.getMembershipEndTime());
        studentHomeVo.setMembershipDays(num < 0 ? 0 : num);

        StudentTotal total = totalService.getTotalById(user.getId());
        if (null == total) {
            total = new StudentTotal();
        }

        studentHomeVo.setExerciseDays(null == total.getExerciseDays() ? 0 : total.getExerciseDays());
        studentHomeVo.setExerciseHours(null == total.getExerciseHours() ? 0 : total.getExerciseHours());
        studentHomeVo.setTestingNum(null == total.getTestingNum() ? 0 : total.getTestingNum());
        studentHomeVo.setFinshClassHours(null == total.getFinshHours() ? 0 : total.getFinshHours());
        studentHomeVo.setUnfinshClassHours(null == total.getUnfinshHours() ? 0 : total.getUnfinshHours());
        studentHomeVo.setStarTeacherNum(null == total.getStarTeacherNum() ? 0 : total.getStarTeacherNum());
        studentHomeVo.setMusicAlbumNum(null == total.getMusicAlbumNum() ? 0 : total.getMusicAlbumNum());
        studentHomeVo.setMusicSheetNum(null == total.getMusicSheetNum() ? 0 : total.getMusicSheetNum());

        // IM聊天用户ID
        studentHomeVo.setImUserId(MessageFormat.format("{0}:{1}", String.valueOf(detail.getUserId()), ClientEnum.STUDENT.name()));

        return studentHomeVo;
    }

    @Override
    public Integer setSubject(Long id, String subjectIds) {
        return baseMapper.setSubject(id, subjectIds);
    }

    @Override
    public List<Subject> querySubject(Long userId) {
        List<Subject> subjects = baseMapper.querySubject(userId);
        return subjects;
    }

    @Override
    public List<Subject> querySubjectItem(Long userId, String type) {
        //先查询自己的声部
        List<Subject> subjectList = querySubject(userId);

        List<Subject> cacheByType = getCacheByType(userId, type);
        if (CollectionUtils.isNotEmpty(cacheByType)) {
            subjectList.addAll(cacheByType);
        }
        return subjectList;
    }

    private List<Subject> getCacheByType(Long userId, String type) {
        RMap<String, List<Subject>> map = redissonClient.getMap(CacheNameEnum.STUDENT_SUBJECT_ITEM.getRedisKey(userId));
        //设置过期时间
        map.expire(CacheNameEnum.STUDENT_SUBJECT_ITEM.getDuration().toMinutes(),
                TimeUnit.MINUTES);

        if (StringUtil.isEmpty(type)) {
            List<Subject> subjectList = map.get("ALL");
            if (CollectionUtils.isEmpty(subjectList)) {
                subjectList = baseMapper.querySubjectItem(userId, null);
                map.put("ALL", subjectList);
            }
            return subjectList;
        } else {
            List<Subject> subjectList = map.get(type);
            if (CollectionUtils.isEmpty(subjectList)) {
                subjectList = baseMapper.querySubjectItem(userId, type);
                map.put(type, subjectList);
            }
            return subjectList;
        }
    }

    @Override
    public IPage<MyFollow> queryMyFollow(IPage<MyFollow> page, QueryMyFollowSearch query) {
        List<MyFollow> teacherVos = baseMapper.queryMyFollow(page, query);
        return page.setRecords(teacherVos);
    }

    @Override
    public HttpResponseResult<Map<String, TeacherVo>> bindTeacher(String phone, Long userId, Boolean isUpdate) {
        Map<String, TeacherVo> resMap = new HashMap<>();
        TeacherVo old = userBindingTeacherDao.getBindTeacherByPhone(phone);
        if (null != old) {
            //身份证号、手机号脱敏
            old.setIdCardNo(ValueUtil.fuzzyIdCard(old.getIdCardNo()));
            old.setPhone(ValueUtil.fuzzyMobile(old.getPhone()));
        }
        resMap.put("old", old);

        StudentVo studentVo = detailByPhone(phone);

        if (null == studentVo) {
            return HttpResponseResult.failed("未找到用户信息");
        }
        TeacherVo detail = teacherDao.detail(userId);
        if (null != detail) {
            //身份证号、手机号脱敏
            detail.setIdCardNo(ValueUtil.fuzzyIdCard(detail.getIdCardNo()));
            detail.setPhone(ValueUtil.fuzzyMobile(detail.getPhone()));
        }
        if (null == detail) {
            return HttpResponseResult.failed("未找老师信息");
        }
        if (UserLockFlag.LOCKED.equals(detail.getLockFlag())) {
            return HttpResponseResult.failed("老师被锁定");
        }
        if (null == old || (null != isUpdate && isUpdate)) {
            //更新
            //删除用户绑定
            userBindingTeacherDao.delete(Wrappers.<UserBindingTeacher>lambdaQuery()
                    .eq(UserBindingTeacher::getStudentId, studentVo.getUserId()));

            UserBindingTeacher userBindingTeacher = new UserBindingTeacher();
            userBindingTeacher.setTeacherId(userId);
            userBindingTeacher.setStudentId(studentVo.getUserId());
            userBindingTeacherDao.insert(userBindingTeacher);

            //学生老师增加好友关系
            Set<Long> studentIds = new HashSet<>();
            studentIds.add(studentVo.getUserId());
            imUserFriendService.saveUserFriend(userId, studentIds);
        }
        resMap.put("now", detail);
        return HttpResponseResult.succeed(resMap);
    }

    /**
     * 学生帐号关联
     *
     * @param info StudentWrapper.UnionStudent
     * @return StudentWrapper.UnionStudentResp
     */
    @Transactional
    @Override
    public StudentWrapper.UnionStudentResp unionStudent(StudentWrapper.UnionStudent info) {

        // 帐号关联响应数据
        StudentWrapper.UnionStudentResp studentResp = StudentWrapper.UnionStudentResp
                .builder()
                .unionId(info.getUnionId())
                .mobile(info.getMobile())
                .updateFlag(false)
                .build();

        // 手机号查询用户信息
        StudentVo studentVo = detailByPhone(info.getMobile());


        com.yonge.cooleshow.biz.dal.entity.SysUser sysUser = sysUserMapper.selectOne(Wrappers.<com.yonge.cooleshow.biz.dal.entity.SysUser>lambdaQuery()
                                                                                             .eq(com.yonge.cooleshow.biz.dal.entity.SysUser::getPhone, info.getMobile()));

        studentResp.setSysUser(sysUser);
        if (Objects.isNull(studentVo)) {

            if (Objects.isNull(sysUser)) {

                // 自动注册创建用户
                com.yonge.cooleshow.biz.dal.entity.SysUser user = new com.yonge.cooleshow.biz.dal.entity.SysUser();
                user.setUsername(info.getUsername());
                user.setAvatar("https://daya.ks3-cn-beijing.ksyun.com/202203/T1WgJaE.png");
                user.setPhone(info.getMobile());
                user.setLockFlag(0);
                user.setGender(info.getGender());
                user.setUserType(ClientEnum.STUDENT.getCode());
                sysUserMapper.insert(user);

                studentResp.username(user.getUsername()).userId(user.getId()).setSysUser(user);

                // 更新标识
                studentResp.setUpdateFlag(true);
            } else {

                com.yonge.cooleshow.biz.dal.entity.SysUser updateEntity = new com.yonge.cooleshow.biz.dal.entity.SysUser();
                updateEntity.setId(sysUser.getId());
                updateEntity.setUpdateTime(LocalDateTime.now());
                //updateEntity.setDelFlag(Optional.ofNullable(info.getLockFlag()).orElse(0)); // 激活帐号
                if (StringUtils.isBlank(sysUser.getUsername())) {
                    updateEntity.setUsername(info.getUsername());
                }
                // 激活帐号
                sysUserMapper.updateById(updateEntity);

                studentResp.username(sysUser.getUsername()).userId(sysUser.getId());

                Student student = lambdaQuery().eq(Student::getUserId, sysUser.getId()).one();

                if (Objects.nonNull(student)) {
                    // 重置学生隐藏状态
                    info.setLockFlag(0);
                }
                // 更新学生关联帐号
                updateStudentUnionStatus(info, studentResp, student);

            }

            String subjectId = null;
            // 学生声部
            if (!StringUtil.isEmpty(info.getSubjectName())) {
                Subject subject = subjectDao.getSubjectMatchByName(info.getSubjectName());
                if (Objects.nonNull(subject)) {
                    subjectId = String.valueOf(subject.getId());
                }
            }

            // 添加学生帐号
            Student student = new Student();
            student.setUserId(studentResp.getUserId());
            student.setSubjectId(subjectId);
            student.setUnionId(info.getUnionId());
            student.setLockFlag(UserLockFlag.NORMAL);
            student.setHideFlag(Optional.ofNullable(info.getLockFlag()).orElse(0));
            save(student);

        } else {

            studentResp.username(studentVo.getUsername()).userId(studentVo.getUserId());

            Student student = lambdaQuery().eq(Student::getUserId, studentVo.getUserId()).one();

            // 重置学生隐藏状态
            info.setLockFlag(0);
            // 更新学生关联帐号
            updateStudentUnionStatus(info, studentResp, student);
        }

        return studentResp;
    }

    /**
     * 更新学生关联帐号
     * @param info StudentWrapper.UnionStudent
     * @param studentResp StudentWrapper.UnionStudentResp
     * @param student Student
     */
    private void updateStudentUnionStatus(StudentWrapper.UnionStudent info,
                                          StudentWrapper.UnionStudentResp studentResp, Student student) {

        if (Objects.nonNull(student)) {

            if (Optional.ofNullable(student.getUnionId()).orElse(0L) <= 0) {

                // 更新学生关联Id
                lambdaUpdate()
                        .eq(Student::getUserId, student.getUserId())
                        .set(Student::getUnionId, info.getUnionId())
                        .set(Student::getHideFlag, Optional.ofNullable(info.getLockFlag()).orElse(0))
                        .update();
            } else {
                studentResp.setUnionId(student.getUnionId());
            }

            // 更新标识
            studentResp.setUpdateFlag(true);
        }
    }

}